==> radius-0.1.0.tar.gz: radius0.1.0/util/udpTransport.cxx <==
/*********************************************************************

 $Id: udpTransport.cxx,v 1.20 2000/05/01 19:16:19 bogawa Exp $

 *********************************************************************

  This library is free software; you can redistribute it and/or modify
  it under the terms of the GNU Lesser General Public License as
  published by the Free Software Foundation; either version 2 of the
  License, or (at your option) any later version.  In addition to the
  terms and conditions set forth in the GNU Lesser General Public
  License, as a condition of using this library you are required to
  grant to all users of this library or any implementation utilizing
  or derived from this library a reciprocal, no cost, worldwide,
  perpetual, non-exclusive, non-transferable, unrestricted license to
  your claims of all patents and patent applications throughout the
  world that are infringed by the library or any implementation
  utilizing or derived from this library.  In the event you
  redistribute this library or any implementation utilizing or derived
  from this library, you must prominently display the foregoing terms
  and conditions with the library or the implementation utilizing or
  derived from this library.

  In the event of a conflict of terms between the foregoing license
  grant and the terms set forth in the GNU Lesser General Public
  License, the foregoing terms and conditions shall be deemed to
  govern.

  This library is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not; write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA.

  Copyright 1999-2000 Vovida Networks, Inc.  All Rights Reserved.

 *********************************************************************

 $Log: udpTransport.cxx,v $
 Revision 1.20  2000/05/01 19:16:19  bogawa
 fixes for FreeBSD compiling in util directory

 Revision 1.19  2000/01/11 00:56:01  bogawa
 copyright notice fixes for year 2000

 Revision 1.18  1999/12/21 21:51:46  bogawa
 fixes for the great renaming

 Revision 1.17  1999/11/30 02:09:31  eckelcu
 Modified Files:
 	Data.cxx Data.hxx DataBuff.cxx DataBuff.hxx cpLog.cxx
 	debug.cxx fifo.cxx parse3tuple.cxx parse3tupleTest.cxx
 	sptr.hxx support.cxx support.hxx timer.cxx udp.cxx udp.hxx
 	udpTransport.cxx udpTransport.hxx vcondition-linux.cxx
 	vcondition-vxworks.cxx vmutex-linux.cxx vmutex-vxworks.cxx
 	vthread-linux.cxx vthread-vxworks.cxx vtime-vxworks.cxx
 Added consistent cvs header and LGPL license info to all files in /util

 Revision 1.16  1999/10/25 22:02:20  bogawa
 merge from irix6-0914 back into the main branch

 Revision 1.14.2.3  1999/09/16 23:26:01  bogawa
 merge back fixes from main

 Revision 1.15  1999/09/15 23:55:55  bogawa
 merge in changes from irix6-0914 -- merge-irix6-0914-B01

 Revision 1.14.2.2  1999/09/15 22:15:57  bogawa
 merge in nicer display for message dumps via mgcp-debug

 Revision 1.14.2.1  1999/09/15 00:24:20  bogawa
 merge in cheung's changes and changes to compile under sgi

 Revision 1.14  1999/09/13 20:40:59  bogawa
 changes for async as well as other useful fixes.

 Revision 1.13  1999/08/19 00:07:53  bogawa
 API change
  Modified Files:
  	Makefile commandCoding.cxx commandCoding.hxx eventCoding.cxx
  	eventCoding.hxx parameterCoding.cxx parameterCoding.hxx
  	sdp.cxx sdp.hxx support.cxx support.hxx udpTransport.cxx
  	udpTransport.hxx
  Added Files:
  	debug.cxx debug.h sptr.hxx

 Revision 1.12  1999/07/27 00:23:01  bogawa
 Add Win32 (Visual c++ 6) compilation support

 Revision 1.11  1999/07/23 23:57:25  bogawa
 the mgcp/examples/gateway Makefile was changed to use the new
 make file system.
 improve the global makefiles.  Now, most targets use these makefiles
 which should simplify the lives of the users, we hope.
 Key features include support for nested object directories as well as
 a more coherent install strategy (so that installs are less likely to
 produce error messages)
 in addition, the code was cleaned up to add a few new features and to
 remove some warnings.

 Revision 1.10  1999/07/22 19:10:36  bogawa
 Updated API.  Major changes include threading changes and blocking API
 changes, also added new functionality.

 Revision 1.9  1999/07/09 08:20:03  bogawa
 Updated the code to do RequestedEvents properly.  Some stuff still
 doesn't work, but at least the Actions are now properly sent.
 This changes the exposed API for users, but the changes from
 here on out should be less.

 Revision 1.8  1999/07/08 23:42:09  bogawa
 Reduce number of warnings

 Revision 1.7  1999/06/30 06:09:08  bogawa
 update copyright and license

 Revision 1.6  1999/06/29 23:42:03  bogawa
 Updated copyright notice.

 Revision 1.5  1999/06/28 21:30:37  bogawa
 Update to the latest version of the MGCP stack.

 Revision 1.4  1999/06/09 08:16:44  bogawa
 Add copyright notices.

 Revision 1.3  1999/06/09 01:04:59  bogawa
 Changes to make the new structure compile correctly, and to allow the
 code to use the new hardware structure.  gateway1 is now using the
 new structure fully, while gateway2 is using it partially (actually
 not very much).
 A future version should be much cleaner.

 Revision 1.2  1999/06/08 20:24:53  bogawa
 some more changes to allow for correct building and add the proper tags
 for cvs

 Revision 1.1.1.1  1999/06/08 18:12:25  bogawa
 make file system for cvs

 Revision 1.1.1.1  1999/06/04 23:52:53  bogawa
 create files
    
 ********************************************************************/


#include "StdAfx.h"
#include <sys/types.h>
#ifndef WIN32
#include <sys/socket.h>
#endif
#include <string.h> // for memset 
#ifndef WIN32
#include <netdb.h>
#include <sys/errno.h>
#endif
#include "vovida-endian.h"
#ifdef __sgi
#include <errno.h>
#define socklen_t int
#endif
#include <assert.h>
#include <iostream>

#include "MgcpUdpStack.hxx"
#include "MgcpDebugManager.h"


MgcpUdpPacket::MgcpUdpPacket(int payloadSize)
{
   data=NULL; size=0; allocatedSize=0;

//    assert( payloadSize > 0 );
//    assert( payloadSize < 8192 ); // some udp stacks will have problems with large data 
//    assert( numSource >= 0 );
//    assert( numSource <= 15 );
//    assert( padBytes >= 0 );
//    assert( padBytes < 8192 );

//   assert( sizeof( MgcpUdpHeader ) == 12 );
   
   size = payloadSize;
//sizeof( MgcpUdpHeader ) + 
//       numSource*sizeof(u_int32_t) + payloadSize + padBytes;
   
   data = new char[size];
   assert( data );
   allocatedSize = size;
   
//   memset( data, 0 , sizeof( MgcpUdpHeader ) );

//   MgcpUdpHeader* header = reinterpret_cast<MgcpUdpHeader*>(data);

//   header->version = 2;
//   header->paddingFlag = (padBytes>0)?1:0;
   //header->extetionFlag = 0;
//   header->numSource = numSource;
   //header->markerFlag = 0;
   //header->payloadType = MgcpUdpPayloadUndefined;
   
   //header->seqenceNumber =0;
   //header->timeStamp = 0;
   //header->ssrc = 0;
}


MgcpUdpPacket::~MgcpUdpPacket()
{
   delete [] data; data=NULL;
   size=0; 
   allocatedSize=0;
}


char* MgcpUdpPacket::getMessageLocation()
{
   return data;
}


ostream& MgcpUdpPacket::print(ostream& s) const
{
    s << data;
    return s;
}



int MgcpUdpPacket::getMessageSize()
{
   assert( size > 0 );
   return size;
}


void
MgcpUdpPacket::setMessageSize(int s)
{
   assert( s > 0 );
   assert( s <= allocatedSize );
   
   size = s;
}


MgcpUdpStack::MgcpUdpStack(const char* serverName /* null if this is the server */ ,
		   int aPort )
{
   assert( aPort > 1024 );
   assert( aPort < 65000 );
   
   port = aPort;
   
   //socketFd = socket(AF_INET,SOCK_DGRAM,0);
   socketFd = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
   if ( socketFd < 0  )
   {
      // error 
       cerr << "Unable to bind to socket";  
      assert(0);
   }
   
   memset((char*) &rxAddress,0, sizeof(rxAddress));

   assert( port >= 1024 ); // don't use privliged ports 
   
   rxAddress.sin_family = AF_INET;
   rxAddress.sin_port = htons(port);
   rxAddress.sin_addr.s_addr = htonl(INADDR_ANY);
//   rxAddress.sin_len = sizeof(rxAddress);

   if ( serverName )
   {
      // this is a client

      struct hostent* host = gethostbyname (serverName);
      if (host == NULL)
      {
	 cerr << "Can't lookup hostname " 
	      << serverName
	      << endl;
	 
	 throw MgcpUdpException(MgcpUdpErrorUnkownHost);
	 
	 assert(0);
      }
      else
      {
	 assert( host->h_addr_list[0] );
	 memcpy((char*)&rxAddress.sin_addr.s_addr,
		host->h_addr_list[0], // take the first entry 
		host->h_length);
      }

      if (connect(socketFd,
		  (struct sockaddr*)  &rxAddress,
		  sizeof(rxAddress)))
      {
	  cerr << "Unable to connect to socket";
	 assert(0);
      }
   }
   else
   {
      // this is a server 

      if ( bind( socketFd, 
		 (struct sockaddr*) &rxAddress,
		 sizeof(rxAddress))
	   != 0 )
      {
	  cerr << "Unable to bind to rx socket -- is another server running?\n";
	 assert(0);
      }
   }
}


// MgcpUdpReceiver::MgcpUdpReceiver( int aPort=5004 , int jitterBufferMs = 50 ) :
MgcpUdpReceiver::MgcpUdpReceiver( int aPort, int jitterBufferMs) :
   MgcpUdpStack( NULL , aPort )
{
}


int 
MgcpUdpReceiver::receive( char* buffer, int bufferSize)
{
   // form a RTP packet, do normal receive, copy data to buffer, free packet 
   return 0;
}


MgcpUdpPacket* 
MgcpUdpReceiver::receive()
{
   /* 
      This function needs to do much of the work for the stack. 

      First all packets in the OS que for the port must be recieved
      Packets must be reordered. 
      Duplicate packets must be chucked out. 
      
      Next the jitter buffer control is implemented. If a packet is deamed to be 
      still younger than the jitter buffer time, it is NOT returned to the user
      until it is old enough. 

      Missing packets are inserted using the provided algorithm. 

      Packets are translated from network format to api format usings CODECs
      
      Statistics are collected, and may result in transmitting some RTCP info

      Any RTCP info is received and used to update statitics

      Error information is passed up to the user of the stack 
   */

   return getPacket();
}


MgcpUdpPacket* 
MgcpUdpReceiver::getPacket()
{
    /// xxx
   MgcpUdpPacket* packet = new MgcpUdpPacket(512 /* buffer size */ ); // fix hard code here 
   
   //unsigned int fromLen = sizeof(txAddress);
//   int fromLen = sizeof(txAddress);
   socklen_t fromLen = sizeof(txAddress);
//   int fromLen = sizeof(txAddress);
   
   int len = recvfrom( socketFd, 
		       packet->getMessageLocation(), 
		       packet->getMessageSize(),
		       0 /*flags */,
		       (struct sockaddr*) &txAddress,
		       &fromLen);
   
   if ( len <= 0 )
   {
      assert(0);

      delete packet; packet=NULL;
      return NULL;
   }
   
   packet->setMessageSize(len);
   
   assert( fromLen <= sizeof(txAddress) );

   MgcpDumpMessage("IN ", '*', packet);
      
   return packet;
}


void MgcpUdpReceiver::addToFdSet( fd_set* set )
{
    FD_SET(socketFd, set);
}      

bool MgcpUdpReceiver::checkIfSet( fd_set* set )
{
    // return static_cast<bool> FD_ISSET(socketFd, set);
    return FD_ISSET(socketFd, set) != 0;
}


MgcpUdpTransmitter::MgcpUdpTransmitter( const char* destinationHostName,
				int aPort ):
   MgcpUdpStack(destinationHostName,aPort)
{
}


void 
MgcpUdpTransmitter::transmit( char* data, int length )
{
    MgcpUdpPacket packet(length);

    memcpy(packet.getMessageLocation() , data, length);

    transmit(&packet);
}


void
MgcpUdpTransmitter::transmit( MgcpUdpPacket* packet )
{
   /*
     Much work needs to be done in this function. 

     First the packet is translated to network format. 

     Any FEC information is computed and saved for later transmition.

     Previous FEC information is transmitted.

     This packet is transmitted.

     Statistics are updated and may send RTCP packets. 

     RTCP info is received and statistics are update. 

     Error are provided up to the stack user
   */
  
//   assert( packet->getPayloadType() == apiFormat );
   
//   if ( apiFormat != networkFormat )
//   {
      // need to translate packet using CODEC 
//      assert(0);
//   }
   
//   packet->setSequenceNumber( nextSequenceNumber++ );

//   previousTimestamp = packet->getTimestamp();

    MgcpDumpMessage("OUT", '-', packet);
   
   int count = sendto( socketFd, 
	   packet->getMessageLocation(), packet->getMessageSize(),
	   0 /* flags */ ,
	   (struct sockaddr*)  0, 0);
#if 0
int count = sendto( socketFd, 
	   packet->getMessageLocation(), packet->getMessageSize(),
	   0 /* flags */ ,
	   (struct sockaddr*)  &rxAddress, sizeof(rxAddress) );
#endif
	if(count < 0)
	{
	    // an error -- but what?
	    cerr << "Udp send error: " << errno << "\n";
	}
}


MgcpUdpPacket* 
MgcpUdpTransmitter::allocatePacket( int payloadLength )
{
   MgcpUdpPacket* packet = new MgcpUdpPacket( payloadLength );
   assert( packet );
   
//   packet->setSSRC(SSRC);
//   packet->setPayloadType( apiFormat );
   
   return packet;
}


MgcpUdpException::MgcpUdpException( MgcpUdpErrorType theErrorType ):
   errorType(theErrorType)
{
}

==> radius-1.0.0.tar.gz: radius1.0.0/util/udpTransport.cxx <==

/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 1999, 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by Vovida Networks,
 *        Inc. (http://www.vovida.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must 
 *    not be used to endorse or promote products derived from this 
 *    software without prior written permission. For written permission, 
 *    please contact vocal@vovida.org.
 *
 * 5. Products derived from this software may not be called "VOCAL",
 *    nor may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL VOVIDA NETWORKS, INC. OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida 
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see 
 * <http://www.vovida.org/>.
 *
 */





static const char* const udpTransport_cxx_Version = 
"$Id: udpTransport.cxx,v 1.22 2000/08/10 11:16:59 bogawa Exp $";



#include "StdAfx.h"
#include <sys/types.h>
#ifndef WIN32
#include <sys/socket.h>
#endif
#include <string.h> // for memset 
#ifndef WIN32
#include <netdb.h>
#include <sys/errno.h>
#endif
#include "vovida-endian.h"
#ifdef __sgi
#include <errno.h>
#define socklen_t int
#endif
#include <assert.h>
#include <iostream>

#include "MgcpUdpStack.hxx"
#include "MgcpDebugManager.h"


MgcpUdpPacket::MgcpUdpPacket(int payloadSize)
{
   data=NULL; size=0; allocatedSize=0;

//    assert( payloadSize > 0 );
//    assert( payloadSize < 8192 ); // some udp stacks will have problems with large data 
//    assert( numSource >= 0 );
//    assert( numSource <= 15 );
//    assert( padBytes >= 0 );
//    assert( padBytes < 8192 );

//   assert( sizeof( MgcpUdpHeader ) == 12 );
   
   size = payloadSize;
//sizeof( MgcpUdpHeader ) + 
//       numSource*sizeof(u_int32_t) + payloadSize + padBytes;
   
   data = new char[size];
   assert( data );
   allocatedSize = size;
   
//   memset( data, 0 , sizeof( MgcpUdpHeader ) );

//   MgcpUdpHeader* header = reinterpret_cast<MgcpUdpHeader*>(data);

//   header->version = 2;
//   header->paddingFlag = (padBytes>0)?1:0;
   //header->extetionFlag = 0;
//   header->numSource = numSource;
   //header->markerFlag = 0;
   //header->payloadType = MgcpUdpPayloadUndefined;
   
   //header->seqenceNumber =0;
   //header->timeStamp = 0;
   //header->ssrc = 0;
}


MgcpUdpPacket::~MgcpUdpPacket()
{
   delete [] data; data=NULL;
   size=0; 
   allocatedSize=0;
}


char* MgcpUdpPacket::getMessageLocation()
{
   return data;
}


ostream& MgcpUdpPacket::print(ostream& s) const
{
    s << data;
    return s;
}



int MgcpUdpPacket::getMessageSize()
{
   assert( size > 0 );
   return size;
}


void
MgcpUdpPacket::setMessageSize(int s)
{
   assert( s > 0 );
   assert( s <= allocatedSize );
   
   size = s;
}


MgcpUdpStack::MgcpUdpStack(const char* serverName /* null if this is the server */ ,
		   int aPort )
{
    assert(0);
   assert( aPort > 1024 );
   assert( aPort < 65000 );
   
   port = aPort;
   
   //socketFd = socket(AF_INET,SOCK_DGRAM,0);
   socketFd = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
   if ( socketFd < 0  )
   {
      // error 
       cerr << "Unable to bind to socket";  
      assert(0);
   }
   
   memset((char*) &rxAddress,0, sizeof(rxAddress));

   assert( port >= 1024 ); // don't use privliged ports 
   
   rxAddress.sin_family = AF_INET;
   rxAddress.sin_port = htons(port);
   rxAddress.sin_addr.s_addr = htonl(INADDR_ANY);
//   rxAddress.sin_len = sizeof(rxAddress);

   if ( serverName )
   {
      // this is a client

//      struct hostent* host = gethostbyname (serverName);
       assert(0);  // this code shouldn't be in use anymore
      struct hostent host;
      gethostbyname (serverName);
      if (host == NULL)
      {
	 cerr << "Can't lookup hostname " 
	      << serverName
	      << endl;
	 
	 throw MgcpUdpException(MgcpUdpErrorUnkownHost);
	 
	 assert(0);
      }
      else
      {
	 assert( host->h_addr_list[0] );
	 memcpy((char*)&rxAddress.sin_addr.s_addr,
		host->h_addr_list[0], // take the first entry 
		host->h_length);
      }

      if (connect(socketFd,
		  (struct sockaddr*)  &rxAddress,
		  sizeof(rxAddress)))
      {
	  cerr << "Unable to connect to socket";
	 assert(0);
      }
   }
   else
   {
      // this is a server 

      if ( bind( socketFd, 
		 (struct sockaddr*) &rxAddress,
		 sizeof(rxAddress))
	   != 0 )
      {
	  cerr << "Unable to bind to rx socket -- is another server running?\n";
	 assert(0);
      }
   }
}


// MgcpUdpReceiver::MgcpUdpReceiver( int aPort=5004 , int jitterBufferMs = 50 ) :
MgcpUdpReceiver::MgcpUdpReceiver( int aPort, int jitterBufferMs) :
   MgcpUdpStack( NULL , aPort )
{
}


int 
MgcpUdpReceiver::receive( char* buffer, int bufferSize)
{
   // form a RTP packet, do normal receive, copy data to buffer, free packet 
   return 0;
}


MgcpUdpPacket* 
MgcpUdpReceiver::receive()
{
   /* 
      This function needs to do much of the work for the stack. 

      First all packets in the OS que for the port must be recieved
      Packets must be reordered. 
      Duplicate packets must be chucked out. 
      
      Next the jitter buffer control is implemented. If a packet is deamed to be 
      still younger than the jitter buffer time, it is NOT returned to the user
      until it is old enough. 

      Missing packets are inserted using the provided algorithm. 

      Packets are translated from network format to api format usings CODECs
      
      Statistics are collected, and may result in transmitting some RTCP info

      Any RTCP info is received and used to update statitics

      Error information is passed up to the user of the stack 
   */

   return getPacket();
}


MgcpUdpPacket* 
MgcpUdpReceiver::getPacket()
{
    /// xxx
   MgcpUdpPacket* packet = new MgcpUdpPacket(512 /* buffer size */ ); // fix hard code here 
   
   //unsigned int fromLen = sizeof(txAddress);
//   int fromLen = sizeof(txAddress);
   socklen_t fromLen = sizeof(txAddress);
//   int fromLen = sizeof(txAddress);
   
   int len = recvfrom( socketFd, 
		       packet->getMessageLocation(), 
		       packet->getMessageSize(),
		       0 /*flags */,
		       (struct sockaddr*) &txAddress,
		       &fromLen);
   
   if ( len <= 0 )
   {
      assert(0);

      delete packet; packet=NULL;
      return NULL;
   }
   
   packet->setMessageSize(len);
   
   assert( fromLen <= sizeof(txAddress) );

   MgcpDumpMessage("IN ", '*', packet);
      
   return packet;
}


void MgcpUdpReceiver::addToFdSet( fd_set* set )
{
    FD_SET(socketFd, set);
}      

bool MgcpUdpReceiver::checkIfSet( fd_set* set )
{
    // return static_cast<bool> FD_ISSET(socketFd, set);
    return FD_ISSET(socketFd, set) != 0;
}


MgcpUdpTransmitter::MgcpUdpTransmitter( const char* destinationHostName,
				int aPort ):
   MgcpUdpStack(destinationHostName,aPort)
{
}


void 
MgcpUdpTransmitter::transmit( char* data, int length )
{
    MgcpUdpPacket packet(length);

    memcpy(packet.getMessageLocation() , data, length);

    transmit(&packet);
}


void
MgcpUdpTransmitter::transmit( MgcpUdpPacket* packet )
{
   /*
     Much work needs to be done in this function. 

     First the packet is translated to network format. 

     Any FEC information is computed and saved for later transmition.

     Previous FEC information is transmitted.

     This packet is transmitted.

     Statistics are updated and may send RTCP packets. 

     RTCP info is received and statistics are update. 

     Error are provided up to the stack user
   */
  
//   assert( packet->getPayloadType() == apiFormat );
   
//   if ( apiFormat != networkFormat )
//   {
      // need to translate packet using CODEC 
//      assert(0);
//   }
   
//   packet->setSequenceNumber( nextSequenceNumber++ );

//   previousTimestamp = packet->getTimestamp();

    MgcpDumpMessage("OUT", '-', packet);
   
   int count = sendto( socketFd, 
	   packet->getMessageLocation(), packet->getMessageSize(),
	   0 /* flags */ ,
	   (struct sockaddr*)  0, 0);
#if 0
int count = sendto( socketFd, 
	   packet->getMessageLocation(), packet->getMessageSize(),
	   0 /* flags */ ,
	   (struct sockaddr*)  &rxAddress, sizeof(rxAddress) );
#endif
	if(count < 0)
	{
	    // an error -- but what?
	    cerr << "Udp send error: " << errno << "\n";
	}
}


MgcpUdpPacket* 
MgcpUdpTransmitter::allocatePacket( int payloadLength )
{
   MgcpUdpPacket* packet = new MgcpUdpPacket( payloadLength );
   assert( packet );
   
//   packet->setSSRC(SSRC);
//   packet->setPayloadType( apiFormat );
   
   return packet;
}


MgcpUdpException::MgcpUdpException( MgcpUdpErrorType theErrorType ):
   errorType(theErrorType)
{
}
